local rectshadow do
	rectshadow={}
	local setmt=setmetatable
	local unpack=unpack

	local getshader=love.graphics.getShader
	local getcanvas=love.graphics.getCanvas
	local setshader=love.graphics.setShader
	local setcanvas=love.graphics.setCanvas
	local draw=love.graphics.draw

	local nrects=0
	local rects={}
	local htab={}
	local ptab={}

	function rectshadow.new(h,px,py,sx,sy)
		local rect={
			update=true;
			i=0;
			h=h or 0;
			px=px or 0;
			py=py or 0;
			sx=sx or 0;
			sy=sy or 0;
		}

		local meta={}
		local funcs={}

		function meta:__newindex(i,v)
			rect[i]=v
			rect.update=true
		end
		meta.__index=rect

		local removed=false
		function funcs:remove()
			if not removed then
				removed=true
				local i=rect.i
				rects[nrects].i=i
				rects[i]=rects[nrects]
				htab[i]=htab[nrects]
				ptab[i]=ptab[nrects]
				nrects=nrects-1
			end
		end

		nrects=nrects+1
		rects[nrects]=rect
		htab[nrects]=h or 0
		ptab[nrects]={
			px or 0,py or 0,
			(px or 0)+(sx or 0),
			(py or 0)+(sy or 0)
		}
		rect.i=nrects

		return setmt(funcs,meta)
	end

	local uniformshadowshader=love.graphics.newShader([[
		extern number w;
		extern number h;
		//Must be sorted based on height from least to greatest
		extern float height[64];
		//Stores upper left corner in xy, lower right corner in zw
		extern vec4 pos[64];
		extern number n;

		//idk lol.
		float lightfromarea(in float x0,in float y0,in float x1,in float y1){
			float sx0=inversesqrt(1+x0*x0);
			float sy0=inversesqrt(1+y0*y0);
			float sx1=inversesqrt(1+x1*x1);
			float sy1=inversesqrt(1+y1*y1);
			float atx0=atan(sx0*y0)-atan(sx0*y1);
			float atx1=atan(sx1*y1)-atan(sx1*y0);
			float aty0=atan(sy0*x0)-atan(sy0*x1);
			float aty1=atan(sy1*x1)-atan(sy1*x0);
			return 0.15915494*(sx0*x0*atx0+sx1*x1*atx1+sy0*y0*aty0+sy1*y1*aty1);
		}

		//As long as stuff doesn't overlap, it's exactly correct.
		vec4 effect(vec4 colorin,Image texturein,vec2 posin, vec2 pixelin){
			float px=w*posin.x;
			float py=h*posin.y;

			float besth=0;

			for(int i=0;i<n;i++){
				if(
					besth<height[i]
					&&pos[i].x<=px
					&&pos[i].y<=py
					&&px<=pos[i].z
					&&py<=pos[i].w
				){
					besth=height[i];
				}
			}

			float br=1;

			for(int i=0;i<n;i++){
				float dheight=height[i]-besth;
				if(0<dheight){
					br-=lightfromarea(
						(pos[i].x-px)/dheight,
						(pos[i].y-py)/dheight,
						(pos[i].z-px)/dheight,
						(pos[i].w-py)/dheight
					);
				}
			}

			return max(0,br)*Texel(texturein,posin);
		}
	]])

	function rectshadow.draw(incanvas,outcanvas)
		uniformshadowshader:send("n",nrects)
		if 0<nrects then
			for i=1,nrects do
				local rect=rects[i]
				if rect.update then
					rect.update=false
					htab[i]=rect.h
					local p=ptab[i]
					p[1]=rect.px
					p[2]=rect.py
					p[3]=rect.px+rect.sx
					p[4]=rect.py+rect.sy
				end
			end
			--Is there a less shitty way of doing this?
			--I'd like to do :send("height[5]",x)
			uniformshadowshader:send("height",unpack(htab))
			uniformshadowshader:send("pos",unpack(ptab))
		end

		local w,h=incanvas:getDimensions()
		uniformshadowshader:send("w",w)
		uniformshadowshader:send("h",h)

		local lastshader=getshader()
		local lastcanvas=getcanvas()

		setshader(uniformshadowshader)
		setcanvas(outcanvas)
		draw(incanvas)

		setshader(lastshader)
		setcanvas(lastcanvas)
	end
end











--Now trash code. 








local setcolor do
	local set=love.graphics.setColor
	function setcolor(r,g,b,a)
		set(255*(r or 1),
			255*(g or 1),
			255*(b or 1),
			255*(a or 1))
	end
end

--crappy colorgrad
local tau=2*math.pi
local cos=math.cos
local function grad(g)
	return 0.5*(cos(tau*g)+1),
		0.5*(cos(tau*g+1/3*tau)+1),
		0.5*(cos(tau*g+2/3*tau)+1)
end











local lolcanvas=love.graphics.newCanvas(1024,1024)
love.graphics.setCanvas(lolcanvas)
love.graphics.rectangle("fill",0,0,1024,1024)
love.graphics.setCanvas()

local rects={}

for i=1,64 do
	rects[i]=rectshadow.new(0,(i-1)*16,0,16,16)
end




local shade=true

function love.mousepressed(x,y,b)
	if b==1 then
		shade=not shade
	end
end



local lelshader=love.graphics.newShader([[
	vec4 effect(vec4 colorin,Image texturein,vec2 posin, vec2 pixelin){
		return Texel(texturein,posin);
	}
]])




function love.draw()
	if shade then
		rectshadow.draw(lolcanvas)
	else
		love.graphics.setShader(lelshader)
		love.graphics.draw(lolcanvas)
		love.graphics.setShader()
	end
end

local t=0
function love.update(dt)
	t=t+dt
	love.graphics.setCanvas(lolcanvas)
	love.graphics.clear(255,255,255,255)
	for i=1,#rects do
		local r=rects[i]
		setcolor(grad((i-1)/64))
		r.py=256*math.cos(t+i*6.28318/64)+512
		r.h=64*math.sin(t+i*6.28318/64)+64
		love.graphics.rectangle("fill",r.px,r.py,16,16)
	end
	love.graphics.setCanvas()
end